"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var LogStore_1 = require("./LogStore");
/**
 * Uses different strategies to submit logs to log server via Service facade.
 */
var AdvancedLogger = /** @class */ (function () {
    function AdvancedLogger(configuration) {
        this.configuration = configuration;
        this.logStore = new LogStore_1.default({ transformations: configuration.transformations });
        this.strategy = this.configuration.strategy;
        this.service = this.configuration.service;
        // todo Where is it better to subscribe?
        this.logStore.eventEmitter.on("add", this.onAdd.bind(this));
        this.logStore.eventEmitter.on("clear", this.onClear.bind(this));
        this.logStore.eventEmitter.on("error", this.onStoreError.bind(this));
        this.strategy.eventEmitter.on("send", this.onSend.bind(this));
        this.strategy.eventEmitter.on("error", this.onStrategyError.bind(this));
    }
    AdvancedLogger.prototype.log = function (log) {
        this.logStore.add(log);
    };
    /**
     * Forces strategy to initiate logs sending
     */
    AdvancedLogger.prototype.sendAllLogs = function () {
        this.strategy.sendAll();
    };
    AdvancedLogger.prototype.destroy = function () {
        this.logStore.destroy();
        this.logStore = null;
        this.strategy.destroy();
        this.strategy = null;
        this.service.destroy();
        this.service = null;
        this.configuration = null;
    };
    AdvancedLogger.prototype.onStoreError = function (error) {
        console.error(error);
    };
    AdvancedLogger.prototype.onAdd = function (info) {
        this.strategy.onAdd(info);
    };
    AdvancedLogger.prototype.onClear = function (info) {
        this.strategy.onClear();
    };
    AdvancedLogger.prototype.onStrategyError = function (error) {
        console.error(error);
    };
    AdvancedLogger.prototype.onSend = function () {
        var logs = this.logStore.getAll();
        // We need to clear store as soon as we received request to send all logs
        this.logStore.clear();
        this.service.sendAllLogs(logs)
            .catch(function (error) {
            console.log(error);
            // todo Retry sending logs here or in the service
        });
    };
    return AdvancedLogger;
}());
exports.default = AdvancedLogger;
//# sourceMappingURL=AdvancedLogger.js.map