"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var throttle = require("lodash/throttle");
var TransformationEnum_1 = require("./enums/TransformationEnum");
var LogUtils_1 = require("./util/LogUtils");
var LogStore = /** @class */ (function () {
    function LogStore(config) {
        this.groupLeftIndex = -1;
        this.logs = [];
        this.eventEmitter = new events_1.EventEmitter();
        this.config = config;
        this.initTransformations();
    }
    LogStore.prototype.add = function (log) {
        //todo If has a grouping transformation
        if (this.identityMap) {
            var id = LogUtils_1.default.getLogIdByFields(log, this.groupableConfig.configuration.groupIdentityFields);
            if (!this.identityMap.has(id)) {
                this.logs.push(log);
            }
            this.onAddToGroup(log);
        }
        else {
            this.logs.push(log);
        }
        this.eventEmitter.emit("add", {
            logCount: this.size()
        });
    };
    LogStore.prototype.clear = function () {
        this.logs.length = 0;
        this.eventEmitter.emit("clear", null);
    };
    LogStore.prototype.getAll = function () {
        if (this.throttledOnGroupFinalize) {
            this.throttledOnGroupFinalize.flush();
        }
        return this.logs.slice();
    };
    LogStore.prototype.size = function () {
        return this.logs.length;
    };
    /*public getRealLogCount(): number {
        // todo Take into account the "grouped" counter. Optimize calculation speed using caching in add method
        return this.logs.length;
    }*/
    LogStore.prototype.destroy = function () {
        if (this.throttledOnGroupFinalize) {
            this.throttledOnGroupFinalize.cancel();
        }
        this.logs.length = 0;
        this.eventEmitter.removeAllListeners();
        this.eventEmitter = null;
    };
    //todo Should it be moved to a separate class? Come up with a nicer design
    LogStore.prototype.initTransformations = function () {
        if (this.config && this.config.transformations) {
            var groupableConfig = this.config.transformations
                .find(function (value) { return value.type === TransformationEnum_1.TransformationEnum.RAPID_FIRE_GROUPING; });
            if (groupableConfig) {
                this.groupableConfig = groupableConfig;
                this.identityMap = new Map();
                this.throttledOnGroupFinalize = throttle(this.onGroupFinalize.bind(this), this.groupableConfig.configuration.interval, { trailing: true, leading: false });
                //todo Maybe, move to add method
                this.eventEmitter.on("add", this.throttledOnGroupFinalize);
            }
        }
    };
    LogStore.prototype.onAddToGroup = function (log) {
        var logId = LogUtils_1.default.getLogIdByFields(log, this.groupableConfig.configuration.groupIdentityFields);
        if (this.identityMap.has(logId)) {
            var savedCounter = this.identityMap.get(logId);
            this.identityMap.set(logId, savedCounter + 1);
        }
        else {
            this.identityMap.set(logId, 1);
        }
    };
    LogStore.prototype.onGroupFinalize = function () {
        // Apply grouping counters from Map to logs range from left index to the last
        var len = this.logs.length;
        if (len > 0) {
            var groupConfig = this.groupableConfig.configuration;
            for (var i = this.groupLeftIndex !== -1 ? this.groupLeftIndex : 0; i < len; i++) {
                var log = this.logs[i];
                var id = LogUtils_1.default.getLogIdByFields(log, groupConfig.groupIdentityFields);
                log[groupConfig.groupFieldName] = this.identityMap.has(id) ? this.identityMap.get(id) : 1;
            }
        }
        this.groupLeftIndex = len > 0 ? len - 1 : -1;
        this.identityMap.clear();
    };
    return LogStore;
}());
exports.default = LogStore;
//# sourceMappingURL=LogStore.js.map