"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fast_safe_stringify_1 = require("fast-safe-stringify");
var http_1 = require("./../util/http");
var SumologicService = /** @class */ (function () {
    function SumologicService(config) {
        this.serviceConfig = config.serviceConfig;
        this.defaultLogConfig = config.defaultLogConfig || {};
    }
    // todo Is an additional config needed for initialization?
    // todo When should it be called?
    SumologicService.prototype.initialize = function (config) {
        return Promise.resolve();
    };
    SumologicService.prototype.sendAllLogs = function (logs) {
        var _this = this;
        return this.preparePayload(logs)
            .then(function (payload) {
            if (_this.serviceConfig.retryAttempts && _this.serviceConfig.retryAttempts > 0) {
                return http_1.default.postRequest(_this.serviceConfig, payload)
                    .catch(function () { return _this.retry(_this.serviceConfig.retryAttempts, _this.serviceConfig.retryInterval, http_1.default.postRequest.bind(_this, _this.serviceConfig, payload)); });
            }
            else {
                return http_1.default.postRequest(_this.serviceConfig, payload);
            }
        });
    };
    SumologicService.prototype.preparePayload = function (logs) {
        var _this = this;
        var resultList = logs.map(function (log) {
            var preparedLog = Object.assign({}, _this.defaultLogConfig, log);
            return _this.tryJSONStringify(preparedLog) || fast_safe_stringify_1.default(preparedLog);
        });
        return Promise.resolve(resultList.join("\n"));
    };
    SumologicService.prototype.destroy = function () {
        this.serviceConfig = null;
        this.defaultLogConfig = null;
    };
    SumologicService.prototype.retry = function (retries, delay, fn) {
        var _this = this;
        if (delay === void 0) { delay = 0; }
        return new Promise(function (resolve, reject) {
            setTimeout(function () { return fn().then(resolve).catch(reject); }, delay);
        })
            .catch(function (error) { return retries > 1 ? _this.retry(retries - 1, delay, fn) : Promise.reject(error); });
    };
    /**
     * It is necessary to convert objects safely, otherwise we can lost the whole log bundle
     */
    SumologicService.prototype.tryJSONStringify = function (obj) {
        try {
            return JSON.stringify(obj);
        }
        catch (_) {
            //ignore the error
            return "";
        }
    };
    return SumologicService;
}());
exports.default = SumologicService;
//# sourceMappingURL=SumologicService.js.map