"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var debounce = require("lodash/debounce");
var OnIntervalStrategy = /** @class */ (function () {
    function OnIntervalStrategy(config) {
        /**
         * @type {number}
         */
        this.SEND_INTERVAL = 15000;
        this.eventEmitter = new events_1.EventEmitter();
        if (config.interval) {
            this.SEND_INTERVAL = config.interval;
        }
        this.debouncedSend = debounce(this.send.bind(this), this.SEND_INTERVAL);
    }
    OnIntervalStrategy.prototype.onAdd = function (info) {
        if (info && info.logCount > 0) {
            this.debouncedSend();
        }
    };
    OnIntervalStrategy.prototype.onClear = function () {
        // Ignore log list change
        //console.log("OnIntervalStrategy#cleared");
    };
    OnIntervalStrategy.prototype.sendAll = function (info) {
        this.eventEmitter.emit("send");
    };
    OnIntervalStrategy.prototype.destroy = function () {
        this.eventEmitter.removeAllListeners();
        this.eventEmitter = null;
    };
    OnIntervalStrategy.prototype.send = function () {
        this.eventEmitter.emit("send");
    };
    return OnIntervalStrategy;
}());
exports.default = OnIntervalStrategy;
//# sourceMappingURL=OnIntervalStrategy.js.map